/******************************************
 *
 * $GAMGI/src/gtk/gamgi_gtk_area.c
 *
 * Copyright (C) 2010,2014 Carlos Pereira
 *
 * Distributed under the terms of the GNU
 * General Public License: $GAMGI/LICENSE
 *
 */

#include "gamgi_engine.h"
#include <gdk/gdkx.h>

/********* external function ******
 *                                *
 *       GAMGI_GTK_AREA_QUERY     *
 *                                *
 *  Check if OpenGL is supported  *
 *                                *
 **********************************/

int gamgi_gtk_area_query (void)
{
Display *xdisplay;

/*
 * 1) get default X display;
 * 2) check if OpenGL is supported;
 * 3) return the result as TRUE or FALSE.
 */

xdisplay = gdk_x11_get_default_xdisplay ();
if (xdisplay == NULL || glXQueryExtension (xdisplay,
NULL, NULL) == FALSE) return FALSE;

return TRUE;
}

/************** external function *************
 *                                            *
 *             GAMGI_GTK_AREA_CREATE          *
 *                                            *
 * create OpenGL context for GTK drawing area *
 *                                            *
 **********************************************/

GLXContext gamgi_gtk_area_create (GtkWidget *area,
int *attributes, GLXContext context, int direct)
{
XVisualInfo *xvisual;
Display *xdisplay;
int xscreen;

/************************************************
 * disable GTK double buffer for drawing area   *
 *                                              *
 * get default X display, X screen and X visual *
 ************************************************/

gtk_widget_set_double_buffered (area, FALSE);

xdisplay = gdk_x11_get_default_xdisplay ();
xscreen = DefaultScreen (xdisplay);
xvisual = glXChooseVisual (xdisplay, xscreen, attributes);

/*************************************************************
 * this code works fine but generates a gcc warning saying   *
 * it is redundant and experience shows is is unnecessary    *
 *                                                           *
 * GdkScreen *screen = gdk_screen_get_default ();            *
 * GdkVisual *visual =                                       *
 * gdk_x11_screen_lookup_visual (screen, xvisual->visualid); *
 * gtk_widget_set_visual (area, visual);                     *
 *************************************************************/

/*******************************************************************
 * 1) first OpenGL area (context = NULL): create suitable GL       *
 * context or return NULL                                          *
 * 2) other OpenGL areas (context != NULL): make sure the supplied *
 * context exists at this stage! typically get it from a window    *
 * that exists now, instead of using the context of the first      *
 * window created, which may no longer exist at this stage!        *
 * 3) use direct rendering (TRUE) or rendering through X (FALSE)   *
 *******************************************************************/

return glXCreateContext (xdisplay, xvisual, context, direct);
}

/************** external function *************
 *                                            *
 *             GAMGI_GTK_AREA_REMOVE          *
 *                                            *
 *  Free OpenGL context for GTK drawing area  *
 *                                            *
 **********************************************/

void gamgi_gtk_area_remove (GtkWidget *area, GLXContext context)
{
GdkWindow *drawable;
Display *xdisplay;

/**********************************************
 * 1) get drawable from the GTK drawing area; *
 * 2) get X display from the drawable;        *
 * 3) remove OpenGL context;                  *
 **********************************************/

drawable = gtk_widget_get_window (area);
xdisplay = gdk_x11_drawable_get_xdisplay (drawable);

glXDestroyContext (xdisplay, context);
}

/******** external function *******
 *                                *
 *     GAMGI_GTK_AREA_CURRENT     *
 *                                *
 *   Set current OpenGL context   *
 *                                *
 **********************************/

int gamgi_gtk_area_current (GtkWidget *area, GLXContext context)
{
GdkWindow *drawable;
Display *xdisplay;
Window xid;

/*
 * 1) get drawable from the GTK drawing area;
 * 2) get X display and X window id from the drawable;
 * 3) use X display and X window id to connect with a
 * OpenGL context, to set the current OpenGL rendering context;
 * 4) return TRUE if successful, FALSE otherwise, in which
 * case the previous OpenGL rendering context remains active.
 */

drawable = gtk_widget_get_window (area);
xdisplay = gdk_x11_drawable_get_xdisplay (drawable);
xid = gdk_x11_drawable_get_xid (drawable);

return glXMakeCurrent (xdisplay, xid, context);
}

/********** external function *********
 *                                    *
 *        GAMGI_GTK_AREA_REMOVE       *
 *                                    *
 * Swap front and back OpenGL buffers *
 *                                    *
 **************************************/

void gamgi_gtk_area_swap (GtkWidget *area)
{
GdkWindow *drawable;
Display *xdisplay;
Window xid;

/*
 * 1) get drawable from the GTK drawing area;
 * 2) get X display and X window id from the drawable;
 * 3) use X display and X window id to exchange front
 * and back OpenGL buffers.
 */

drawable = gtk_widget_get_window (area);
xdisplay = gdk_x11_drawable_get_xdisplay (drawable);
xid = gdk_x11_drawable_get_xid (drawable);

glXSwapBuffers (xdisplay, xid);
}

/******* external function ********
 *                                *
 *     GAMGI_GTK_AREA_WAIT_GL     *
 *                                *
 *      Flush OpenGL commands     *
 *                                *
 **********************************/

void gamgi_gtk_area_wait_gl (void)
{
/*
 * 1) wait for completion of all OpenGL tasks before proceding
 */

glXWaitGL();
}

/******* external function ********
 *                                *
 *      GAMGI_GTK_AREA_WAIT_X     *
 *                                *
 *         Flush X commands       *
 *                                *
 **********************************/

void gamgi_gtk_area_wait_x (void)
{
/*
 * 1) wait for completion of all X tasks before proceding
 */

glXWaitX();
}
